package com.foreverwin.mesnac.common.service.impl;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.constant.CustomFieldConstant;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.mapper.ProdReadyTaskMapper;
import com.foreverwin.mesnac.common.model.ProdReadyTask;
import com.foreverwin.mesnac.common.model.ProdReadyTaskDetail;
import com.foreverwin.mesnac.common.service.ProdReadyTaskDetailService;
import com.foreverwin.mesnac.common.service.ProdReadyTaskService;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.meapi.dto.BomComponentDto;
import com.foreverwin.mesnac.meapi.model.ShopOrder;
import com.foreverwin.mesnac.meapi.service.BomComponentService;
import com.foreverwin.mesnac.meapi.service.CustomFieldsService;
import com.foreverwin.mesnac.meapi.service.ShopOrderService;
import com.foreverwin.modular.core.exception.BaseException;
import com.foreverwin.modular.core.util.CommonMethods;
import com.foreverwin.modular.core.util.FrontPage;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 生产准备任务主表 服务实现类
 * </p>
 *
 * @author Philip
 * @since 2021-06-23
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ProdReadyTaskServiceImpl extends ServiceImpl<ProdReadyTaskMapper, ProdReadyTask> implements ProdReadyTaskService {

    public static final String P01="P01";
    public static final String P02="P02";
    public static final String P03="P03";
    public static final String STATUS_CANCEL="C";
    public static final String STATUS_DOING="I";
    public static final String STATUS_FINSH="F";
    public static final String STATUS_WAITING="W";
    @Autowired
    private ProdReadyTaskMapper prodReadyTaskMapper;
    @Autowired
    private ProdReadyTaskDetailService prodReadyTaskDetailService;
    @Autowired
    private BomComponentService bomComponentService;
    @Autowired
    private CustomFieldsService customFieldsService;
    @Autowired
    private ShopOrderService shopOrderService;
    @Override
    public IPage<ProdReadyTask> selectPage(FrontPage<ProdReadyTask> frontPage, ProdReadyTask prodReadyTask) {
        QueryWrapper<ProdReadyTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(prodReadyTask);
        return super.page(frontPage.getPagePlus(), queryWrapper);
    }

    @Override
    public List<ProdReadyTask> selectList(ProdReadyTask prodReadyTask) {
        QueryWrapper<ProdReadyTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(prodReadyTask);
        return super.list(queryWrapper);
    }
    @Override
    public void createTask(String sfcDispatchBO, String dispatchStatus, String shopOrder,String sfc, String operation, String resrce, String stepId, String workCenter, String planStartTime)  {
        //根据站点及派工BO查询数据
        String site = CommonMethods.getSite();
        String user = CommonMethods.getUser();

        Date planTime = null;
        try {
            SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            planTime = dateFormat.parse(planStartTime);
        }
        catch (Exception e) {
            throw new BaseException("工单[" + shopOrder + "]计划开始时间格式错误，请核查！");
        }
        ShopOrder byId = shopOrderService.getById(HandleEnum.SHOP_ORDER.getHandle(site, shopOrder));
        if ("RELEASE".equals(dispatchStatus)) {
            ProdReadyTask prodReadyTask=new ProdReadyTask();
            prodReadyTask.setSite(site);
            prodReadyTask.setTaskNo(StringUtil.createQUID());
            prodReadyTask.setHandle(HandleEnum.PROD_READY_TASK.getHandle(site,prodReadyTask.getTaskNo()));
            prodReadyTask.setSfcDispatchBo(sfcDispatchBO);
            prodReadyTask.setShopOrder(shopOrder);
            prodReadyTask.setWorkCenter(workCenter);
            prodReadyTask.setSfc(sfc);
            prodReadyTask.setOperation(operation);
            prodReadyTask.setStepId(stepId);
            prodReadyTask.setItem(StringUtil.trimHandle(byId.getItemBo()));
            prodReadyTask.setResrce(resrce);
            prodReadyTask.setCreateUser(user);
            prodReadyTask.setCreatedDateTime(LocalDateTime.now());
            prodReadyTask.setModifyUser(user);
            prodReadyTask.setModifiedDateTime(LocalDateTime.now());
            List<ProdReadyTaskDetail> prodReadyTaskDetailList = new ArrayList<>();

            String sfcBo = HandleEnum.SFC.getHandle(site, sfc);
            List<BomComponentDto> bomComponentList = bomComponentService.getSfcBomComponent(sfcBo);
            if (!bomComponentList.isEmpty()){
                bomComponentList.forEach(bomComponentDto ->{
                    if (bomComponentDto.getOperation().equals(operation)&&bomComponentDto.getStepId().equals(stepId)){
                        String accessoryType = bomComponentDto.getAccessoryType();
                        //物料P001，校验当前派工单（SFC+工序/步骤）加工是否需要准备物料
                        if (accessoryType==null||accessoryType.equals("0")){
                            ProdReadyTaskDetail prodReadyTaskDetail=new ProdReadyTaskDetail();
                            prodReadyTaskDetail.setHandle(HandleEnum.PROD_READY_TASK_DETAIL.getHandle(prodReadyTask.getHandle(),P01));
                            prodReadyTaskDetail.setProdReadyTaskBo(prodReadyTask.getHandle());
                            prodReadyTaskDetail.setInspectionItem(P01);
                            prodReadyTaskDetail.setReslut("");
                            prodReadyTaskDetail.setRemarks("");
                            prodReadyTaskDetail.setCreateUser(user);
                            prodReadyTaskDetail.setCreatedDateTime(LocalDateTime.now());
                            prodReadyTaskDetail.setModifyUser(user);
                            prodReadyTaskDetail.setModifiedDateTime(LocalDateTime.now());
                            prodReadyTaskDetailList.add(prodReadyTaskDetail);
                        }else if (accessoryType.equals("1")){
                            //刀具P002，校验当前派工单（SFC+工序/步骤）加工是否需要准备物料
                            ProdReadyTaskDetail prodReadyTaskDetail=new ProdReadyTaskDetail();
                            prodReadyTaskDetail.setHandle(HandleEnum.PROD_READY_TASK_DETAIL.getHandle(prodReadyTask.getHandle(),P02));
                            prodReadyTaskDetail.setProdReadyTaskBo(prodReadyTask.getHandle());
                            prodReadyTaskDetail.setInspectionItem(P02);
                            prodReadyTaskDetail.setReslut("");
                            prodReadyTaskDetail.setRemarks("");
                            prodReadyTaskDetail.setCreateUser(user);
                            prodReadyTaskDetail.setCreatedDateTime(LocalDateTime.now());
                            prodReadyTaskDetail.setModifyUser(user);
                            prodReadyTaskDetail.setModifiedDateTime(LocalDateTime.now());
                            prodReadyTaskDetailList.add(prodReadyTaskDetail);
                        }
                    }
                } );
            }
            //工控程序P003，工控程序，校验是否需要准备工控程序
            String customFieldsValue = customFieldsService.getCustomFieldsValue(HandleEnum.WORK_CENTER.getHandle(site, workCenter), CustomFieldConstant.WORK_CENTER_CUSTOM_FIELD_PROCESS_CONTROL);
            if (StringUtil.notBlank(customFieldsValue)&&customFieldsValue.equals(Constants.STATUS_Y)){
                customFieldsValue = customFieldsService.getCustomFieldsValue(HandleEnum.RESOURCE.getHandle(site, resrce), CustomFieldConstant.RESRCE_CUSTOM_FIELD_PROCESS_CONTROL);
                if (StringUtil.notBlank(customFieldsValue)&&customFieldsValue.equals(Constants.STATUS_Y)) {
                    ProdReadyTaskDetail prodReadyTaskDetail=new ProdReadyTaskDetail();
                    prodReadyTaskDetail.setHandle(HandleEnum.PROD_READY_TASK_DETAIL.getHandle(prodReadyTask.getHandle(),P03));
                    prodReadyTaskDetail.setProdReadyTaskBo(prodReadyTask.getHandle());
                    prodReadyTaskDetail.setInspectionItem(P03);
                    prodReadyTaskDetail.setReslut("");
                    prodReadyTaskDetail.setRemarks("");
                    prodReadyTaskDetail.setCreateUser(user);
                    prodReadyTaskDetail.setCreatedDateTime(LocalDateTime.now());
                    prodReadyTaskDetail.setModifyUser(user);
                    prodReadyTaskDetail.setModifiedDateTime(LocalDateTime.now());
                    prodReadyTaskDetailList.add(prodReadyTaskDetail);
                }
            }
            if (prodReadyTaskDetailList.size()>0){
                prodReadyTask.setStatus(STATUS_WAITING);
                prodReadyTaskDetailService.saveBatch(prodReadyTaskDetailList);
            }else {
                prodReadyTask.setStatus(STATUS_FINSH);
                prodReadyTask.setCompleteUser(user);
                prodReadyTask.setCompleteDateTime(LocalDateTime.now());
            }
            save(prodReadyTask);
        }else if("CANCEL".equals(dispatchStatus)){
            LambdaQueryWrapper<ProdReadyTask> lambdaQueryWrapper=new LambdaQueryWrapper<>();
            lambdaQueryWrapper.eq(ProdReadyTask::getSfcDispatchBo,sfcDispatchBO);
            lambdaQueryWrapper.orderByDesc(ProdReadyTask::getCreatedDateTime);
            List<ProdReadyTask> list = list(lambdaQueryWrapper);
            if (list.size()<1){
                throw new BaseException("根据派工主键未找到产前准备任务");
            }
            if (list.get(0).getStatus().equals(STATUS_CANCEL)){
                throw new BaseException("产前准备任务任务已取消");
            }
            LambdaUpdateWrapper<ProdReadyTask> updateWrapper=new LambdaUpdateWrapper<>();
            updateWrapper.set(ProdReadyTask::getStatus,STATUS_CANCEL);
            updateWrapper.set(ProdReadyTask::getCancelUser,user);
            updateWrapper.set(ProdReadyTask::getCancelDateTime,LocalDateTime.now());
            updateWrapper.eq(ProdReadyTask::getSfcDispatchBo,sfcDispatchBO);
            update(updateWrapper);
        }

    }



}