package com.foreverwin.mesnac.dispatch.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.dispatch.dto.UserResourceDto;
import com.foreverwin.mesnac.dispatch.mapper.UserResourceMapper;
import com.foreverwin.mesnac.dispatch.model.UserResource;
import com.foreverwin.mesnac.dispatch.service.UserResourceService;
import com.foreverwin.mesnac.meapi.model.NwaUser;
import com.foreverwin.modular.core.util.CommonMethods;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 员工资源对应表 服务实现类
 * </p>
 *
 * @author Leon.L
 * @since 2021-06-10
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class UserResourceServiceImpl extends ServiceImpl<UserResourceMapper, UserResource> implements UserResourceService {


    @Autowired
    private UserResourceMapper userResourceMapper;


    @Override
    public List<NwaUser> findEmployeeList(String site, String resource) {
        return userResourceMapper.findEmployeeList(site, resource);
    }

    @Override
    public List<UserResource> findEnableList(String site, String resource, String user) {
        return userResourceMapper.findEnableList(site, resource, user);
    }

    @Override
    public List<UserResourceDto> findAll(String resource, String user) {
        String site = CommonMethods.getSite();
        return userResourceMapper.findAll(site, resource, user);
    }

    @Override
    public void saveAll(List<UserResourceDto> userResourceDtoList) {
        List<UserResource> userResourceList = new ArrayList<>();
        String site = CommonMethods.getSite();
        for (UserResourceDto userResourceDto: userResourceDtoList) {
            UserResource userResource = new UserResource();
            userResource.setSite(site);
            userResource.setResourceBo(HandleEnum.RESOURCE.getHandle(site, userResourceDto.getResrce()));
            userResource.setResrce(userResourceDto.getResrce());
            userResource.setUserId(userResourceDto.getUserId());
            userResource.setTemporaryUser(userResourceDto.getTemporaryUser());
            userResource.setStatus(userResourceDto.getStatus());
            userResource.setCreateUser(CommonMethods.getUser());
            userResource.setHandle("UserResourceBo:"+userResource.getSite()+","+
                    userResource.getResrce()+","+userResource.getUserId());
            if(userResourceMapper.selectById(userResource.getHandle()) == null){
                userResource.setCreatedDateTime(LocalDateTime.now());
            }
            userResource.setModifyUser(CommonMethods.getUser());
            userResource.setModifiedDateTime(LocalDateTime.now());

            userResourceList.add(userResource);
        }
        this.saveOrUpdateBatch(userResourceList);
    }
}