package com.foreverwin.mesnac.equip.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.equip.service.ResourceInspectTaskService;
import com.foreverwin.mesnac.equip.model.ResourceInspectTask;
import java.util.List;

/**
 *
 * @author pavel.Liu
 * @since 2021-06-10
 */
@RestController
@RequestMapping("/Z-RESOURCE-INSPECT-TASK")
public class ResourceInspectTaskController {

    @Autowired
    public ResourceInspectTaskService resourceInspectTaskService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getResourceInspectTaskById(@PathVariable String id) {
        return R.ok( resourceInspectTaskService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getResourceInspectTaskList(ResourceInspectTask resourceInspectTask){
        List<ResourceInspectTask> result;
        QueryWrapper<ResourceInspectTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceInspectTask);
        result = resourceInspectTaskService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<ResourceInspectTask> frontPage, ResourceInspectTask resourceInspectTask){
        IPage result;
        QueryWrapper<ResourceInspectTask> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(resourceInspectTask);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(ResourceInspectTask::getHandle, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getSite, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getTaskNo, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getCategory, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getDescription, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getStatus, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getResourceInspectPlanBo, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getInspectionItemBo, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getResourceBo, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getResult, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getExecutor, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getRemark, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getCreateUser, frontPage.getGlobalQuery())
                .or().like(ResourceInspectTask::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = resourceInspectTaskService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param resourceInspectTask  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ResourceInspectTask resourceInspectTask) {
        return R.ok(resourceInspectTaskService.save(resourceInspectTask));
    }

    /**
     * 修改
     * @param resourceInspectTask  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody ResourceInspectTask resourceInspectTask) {
        return R.ok(resourceInspectTaskService.updateById(resourceInspectTask));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(resourceInspectTaskService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(resourceInspectTaskService.removeByIds(ids));
    }
}