package com.foreverwin.mesnac.integration.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.foreverwin.mesnac.common.constant.Constants;
import com.foreverwin.mesnac.common.enums.HandleEnum;
import com.foreverwin.mesnac.common.util.StringUtil;
import com.foreverwin.mesnac.dispatch.model.ItemBatch;
import com.foreverwin.mesnac.dispatch.service.ItemBatchService;
import com.foreverwin.mesnac.integration.service.InterfaceService;
import com.foreverwin.mesnac.meapi.model.Item;
import com.foreverwin.mesnac.meapi.service.ItemService;
import com.foreverwin.modular.core.exception.BusinessException;
import com.foreverwin.modular.core.meext.MEServices;
import com.sap.me.common.CustomValue;
import com.sap.me.common.MaterialType;
import com.sap.me.common.ObjectReference;
import com.sap.me.common.ProcurementType;
import com.sap.me.productdefinition.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.i18n.LocaleContextHolder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;

/**
 *
 * @author Leon
 * @date 2021-06-16
 *
 * 接口服务类
 *
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class InterfaceServiceImpl implements InterfaceService {

    @Autowired
    private ItemService itemService;
    @Autowired
    private ItemBatchService itemBatchService;


    @Override
    public void createOrUpdateItem(JSONObject jsonObject) throws Exception{
        //数据主键
        String site = jsonObject.getString("SITE");
        //物料
        String item = jsonObject.getString("ITEM");
        //版本:默认值A
        String revision = jsonObject.getString("REVISION");
        revision = StringUtil.notBlank(revision) ? revision : "A";
        //物料描述
        String itemDescription = jsonObject.getString("MAKTX");
        //状态
        String status = jsonObject.getString("STATUS");
        String statusBo = HandleEnum.STATUS.getHandle(site, status);
        //物料类型(FERT(已完成)/ROH(原始)/HALB(半成品)/KMAT(可配置)/INST(安装)/VERP(包装)/FHMI(生产资源/工具)/CSTM(自定义))
        String itemType = jsonObject.getString("ITEM_TYPE");
        //采购类型(M(制造)/P(采购)/B(制造/采购))
        String procurementType = jsonObject.getString("PROCUREMENT_TYPE");
        //批次大小
        String lotSize = jsonObject.getString("LOT_SIZE");
        lotSize = StringUtil.notBlank(lotSize) ? lotSize : "999999";
        //装配时要收集的数据
        String assyDataType = jsonObject.getString(" ASSY_DATA_TYPE");
        //库存接收数据类型
        String invAssyDataType = jsonObject.getString("INV_ASSY_DATA_TYPE");
        //计量单位
        String unitOfMeasure = jsonObject.getString("UNIT_OF_MEASURE");

        //物料标准API
        ItemConfigurationServiceInterface itemServiceInterFace = MEServices.create("com.sap.me.productdefinition", "ItemConfigurationService", site);


        //物料是否存在
        String itemBo = HandleEnum.ITEM.getHandle(site, item, revision);
        Item itemModel = itemService.getById(itemBo);
        if (itemModel == null){
            //创建物料
            ItemConfiguration itemConfiguration = new ItemConfiguration();
            itemConfiguration.setItem(item);
            itemConfiguration.setRevision(revision);
            itemConfiguration.setProcurementType(ProcurementType.fromValue(procurementType));
            itemConfiguration.setLotSize(new BigDecimal(lotSize));
            itemConfiguration.setRef(itemBo);
            itemConfiguration.setCurrentRevision(true);
            itemConfiguration.setUnitOfMeasurement(unitOfMeasure);
            itemConfiguration.setMaterialType(MaterialType.fromValue(itemType));

            //物料描述赋值
            List<ItemTranslation> itemTranslationList = new ArrayList<>();
            ItemTranslation itemTranslation = new ItemTranslation();
            itemTranslation.setLocale("zh");
            itemTranslation.setDescription(itemDescription);
            itemTranslationList.add(itemTranslation);
            itemConfiguration.setItemTranslationList(itemTranslationList);
            itemConfiguration.setStatusRef(statusBo);

            //数量限制为任意整数
            itemConfiguration.setQuantityRestriction(QuantityRestriction.ANY_NUMBER);

            //创建物料
            itemServiceInterFace.createItem(itemConfiguration);
        } else {
            //更新物料
            ObjectReference itemReference = new ObjectReference();
            itemReference.setRef(itemBo);
            ItemFullConfiguration itemFullConfiguration = itemServiceInterFace.readItem(itemReference);


            //物料描述赋值
            ItemFullConfiguration updateItem = itemFullConfiguration;
            updateItem.setUnitOfMeasurement(unitOfMeasure);
            updateItem.setMaterialType(MaterialType.fromValue(itemType));
            updateItem.setProcurementType(ProcurementType.fromValue(procurementType));
            updateItem.setLotSize(new BigDecimal(lotSize));
            updateItem.setStatusRef(statusBo);

            List<ItemTranslation> itemTranslationList = new ArrayList<>();
            ItemTranslation itemTranslation = new ItemTranslation();
            Locale locale = LocaleContextHolder.getLocale();
            itemTranslation.setLocale(locale.getLanguage());
            itemTranslation.setDescription(itemDescription);
            itemTranslationList.add(itemTranslation);
            updateItem.setItemTranslationList(itemTranslationList);

            //更新物料
            itemServiceInterFace.updateItem(updateItem);
        }
    }

    @Override
    public void saveItemBatch(JSONObject jsonObject) {
        String site = jsonObject.getString("SITE");
        String factory = jsonObject.getString("FACTORY");
        JSONArray jsonArray = jsonObject.getJSONArray("PURCHASE_ORDER_LIST");
        if (jsonArray == null || jsonArray.size() <= 0) {
            throw BusinessException.build("接口参数【PURCHASE_ORDER_LIST】不能为空");
        }

        LocalDateTime now = LocalDateTime.now();
        //循环处理 物料批次
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject _jsonObject = jsonArray.getJSONObject(i);
            String item = _jsonObject.getString("ITEM");
            String batch = _jsonObject.getString("BATCH");
            String supplier = _jsonObject.getString("SUPPLIER");

            //查询物料和批次是否已经存在记录
            ItemBatch queryBatch = new ItemBatch();
            queryBatch.setBatch(site);
            queryBatch.setItem(item);
            queryBatch.setBatch(batch);
            List<ItemBatch> itemBatchList = itemBatchService.selectList(queryBatch);
            if (itemBatchList != null && itemBatchList.size() > 0) {
                //更新时间
                ItemBatch itemBatch = itemBatchList.get(0);
                itemBatch.setModifyUser(Constants.SITE_ADMIN);
                itemBatch.setModifiedDateTime(now);
                itemBatchService.updateById(itemBatch);
            } else {
                //新增物料批次数据
                ItemBatch itemBatch = new ItemBatch();
                itemBatch.setHandle(HandleEnum.ITEM_BATCH.getHandle(site, item, batch));
                itemBatch.setSite(site);
                itemBatch.setFactory(factory);
                itemBatch.setItem(item);
                itemBatch.setBatch(batch);
                itemBatch.setSupplier(supplier);
                itemBatch.setLabel(item + "*" + batch);
                itemBatch.setIsPrint("false");
                itemBatch.setCreateUser(Constants.SITE_ADMIN);
                itemBatch.setCreatedDateTime(now);
                itemBatchService.save(itemBatch);
            }
        }
    }
}
